@@ -10,7 +10,7 @@ def test_express_pattern(datadir):
 
     result_keys = [str(i) for i in result.keys()]
 
-    expected_keys = ["`PERSON`在`GPE`的`ORG`读书。", "来一首`歌手名`的歌。"]
+    expected_keys = ["<PERSON>在<GPE>的<ORG>读书。", "来一首<歌手名>的歌。"]
 
     for r, e in zip(result_keys, expected_keys):
         assert e in r
