@@ -7,7 +7,9 @@ import operator
 
 def test_repr__():
     span = Span(0, 9, "entity")
-    assert repr(span) == "Span(0, 9, 'entity', value=None, normal_value=None)"
+    expected = eval(repr(span))
+
+    assert span == expected
 
 def test_eq__():
     a = Span(0, 1, "entity")
