@@ -1,11 +1,11 @@
-from typing import List
+from typing import List, Tuple
 
 from tokenizer_tools.tagset.NER.BILUO import BILUOEncoderDecoder
 from tokenizer_tools.tagset.offset.document import Document
 from tokenizer_tools.tagset.offset.span import Span
 
 
-def offset_to_biluo(sequence: Document) -> List[str]:
+def offset_to_biluo(sequence: Document) -> Tuple[List[str], List[str], List[str]]:
     """
     Convert Sequence object to BILUO string
 
@@ -13,15 +13,29 @@ def offset_to_biluo(sequence: Document) -> List[str]:
     :return: string of BILUO encoding
     """
     encoding = ['O'] * len(sequence.text)
-
     for span in sequence.span_set:
         encoder = BILUOEncoderDecoder(span.entity)
         entity_text = sequence.text[span.start: span.end]
         entity_encoding = encoder.encode(entity_text)
-
         encoding[span.start: span.end] = entity_encoding
 
-    return encoding
+    role = ['O'] * len(sequence.text)
+    for span in sequence.span_set:
+        if span.role is not None:
+            encoder = BILUOEncoderDecoder(span.role)
+            entity_text = sequence.text[span.start: span.end]
+            entity_encoding = encoder.encode(entity_text)
+            role[span.start: span.end] = entity_encoding
+
+    group = ['O'] * len(sequence.text)
+    for span in sequence.span_set:
+        if span.group is not None:
+            encoder = BILUOEncoderDecoder(span.group)
+            entity_text = sequence.text[span.start: span.end]
+            entity_encoding = encoder.encode(entity_text)
+            group[span.start: span.end] = entity_encoding
+
+    return encoding, role, group
 
 
 if __name__ == "__main__":
