@@ -1,9 +1,11 @@
 import tempfile
+import math
 import argparse
+import functools
 import os
 from collections import Counter, defaultdict, OrderedDict
 from contextlib import contextmanager
-
+from typing import List, Any
 
 from flask import Flask, request, render_template, jsonify as raw_jsonify
 
@@ -13,6 +15,17 @@ from tokenizer_tools.tagset.offset.dashboard.es_create import insert_corpus_into
 from tokenizer_tools.tagset.offset.dashboard.es_query import query_docs
 from elasticsearch_dsl.connections import connections
 
+from tokenizer_tools.tagset.offset.dashboard.mistagged_span import MistaggedSpan
+from tokenizer_tools.tagset.offset.dashboard.text_perplexity import (
+    batch_get_text_perplexity,
+    compute_histogram,
+)
+from tokenizer_tools.tagset.offset.document import Document
+
+batch_get_text_perplexity = functools.partial(batch_get_text_perplexity, "")
+
+ms = MistaggedSpan()
+
 es_query_func = {}
 es_insert_func = {}
 
@@ -35,7 +48,8 @@ i = SelfIncreaser()
 
 @contextmanager
 def temp_file_context(f):
-    _, actual_tmp_file = tempfile.mkstemp()
+    orig_filename = f.filename
+    _, actual_tmp_file = tempfile.mkstemp(suffix="_" + orig_filename)
     f.save(actual_tmp_file)
     yield actual_tmp_file
     os.unlink(actual_tmp_file)
@@ -48,8 +62,10 @@ class CorpusDataManager:
         self.corpus_docs_length_distribution = None
         self.corpus_docs_by_length = defaultdict(list)
         self.corpus_docs_by_token = defaultdict(list)
+        self.corpus_docs_edge_to_docs = None
         self.corpus_entity_length = None
         self.corpus_docs_tokens = None
+        self.corpus_docs_text_perplexity = None
         self.corpus_statistics = None
         self.domain = None
         self.entity = None
@@ -64,7 +80,12 @@ class CorpusDataManager:
         self.domain_entity_span_corpus = defaultdict(
             lambda: defaultdict(lambda: defaultdict(list))
         )
+        self.corpus_docs_span_entity_type_size = None
+        self.corpus_docs_span_entity_type_size_to_span = None
         self.span_corpus = defaultdict(list)
+        self.corpus_span_untagged = None
+        self.text_span_corpus = None
+        self.corpus_span_untagged_docs = None
         self.span_entity_corpus = defaultdict(lambda: defaultdict(list))
         self.failure_analysis = defaultdict(
             lambda: [None, None, None, None, None, None]
@@ -179,6 +200,19 @@ class CorpusDataManager:
                 self.span_entity_corpus[span_value][entity_type].append(doc)
         i.print_self()
 
+        # entity_type size: int
+        span_entity_type_size = list()
+        # entity_type size: int => list of doc
+        span_entity_type_size_doc = defaultdict(list)
+        for span_value, entity_type in self.span_entity_corpus.items():
+            entity_size = len(entity_type)
+            span_entity_type_size.append(entity_size)
+            span_entity_type_size_doc[entity_size].append(span_value)
+        distribution = Distribution(Counter(span_entity_type_size))
+        self.corpus_docs_span_entity_type_size = distribution.kv_data
+        self.corpus_docs_span_entity_type_size_to_span = span_entity_type_size_doc
+        i.print_self()
+
         # for entity_type in self.corpus_statistics.entity_types.keys():
         #     data = []
         #     for doc in self.corpus:
@@ -219,7 +253,7 @@ class CorpusDataManager:
             self.corpus_docs_by_length[doc_len].append(doc)
         distribution = Distribution(Counter(docs_length))
         self.corpus_docs_length = distribution.kv_data
-        self.corpus_docs_length_distribution = distribution.precentile
+        self.corpus_docs_length_distribution = distribution.percentile
         i.print_self()
 
         entity_length = []
@@ -231,13 +265,46 @@ class CorpusDataManager:
         i.print_self()
 
         docs_tokens = []
+        docs_text = dict()
         for doc in self.corpus:
             docs_tokens.extend(doc.text)
+            docs_text["".join(doc.text)] = doc
             for token in doc.text:
                 self.corpus_docs_by_token[token].append(doc)
+        docs_text_perplexity = batch_get_text_perplexity(list(docs_text.keys()))
+        docs_text_perplexity = [
+            round(float(math.log(j, 10)), 2) for j in docs_text_perplexity
+        ]
+        docs_perplexity = defaultdict(list)
+        for perplexity, doc in zip(docs_text_perplexity, docs_text.values()):
+            docs_perplexity[perplexity].append(doc)
+        # for better display to human
+        hist, bin_edge, text_edge = compute_histogram(docs_text_perplexity)
+        edge_to_docs = defaultdict(list)
+        for perplexity, docs in docs_perplexity.items():
+            for j, (start, end) in enumerate(zip(bin_edge[:-1], bin_edge[1:])):
+                bin_key = ",".join(map(lambda x: "{:.2f}".format(x), (start, end)))
+                if j == len(bin_edge) - 2:
+                    # last bin
+                    if start <= perplexity <= end:
+                        edge_to_docs[bin_key].extend(docs)
+                else:
+                    if start <= perplexity < end:
+                        edge_to_docs[bin_key].extend(docs)
+        self.corpus_docs_edge_to_docs = edge_to_docs
+        self.corpus_docs_text_perplexity = dict(zip(text_edge, hist))
         self.corpus_docs_tokens = Counter(docs_tokens)
         i.print_self()
 
+        result = []
+        ms.add_spans(["".join(j) for j in corpus_data_manager.span_entity_corpus.keys()])
+        self.corpus_span_untagged_docs = ms.process_corpus(corpus_data_manager.corpus)
+        for k, v in self.corpus_span_untagged_docs.items():
+            result.extend([k] * len(v))
+        self.corpus_span_untagged = Counter(result)
+        self.text_span_corpus = {"".join(k): v for k, v in self.span_corpus.items()}
+        i.print_self()
+
     def build_from_failure_analysis(self, actual_file, expected_file):
         for doc in Corpus.read_from_file(actual_file):
             self.failure_analysis[doc.id][0] = doc
@@ -292,7 +359,7 @@ class Distribution:
         self.data = data
 
         self.kv_data = self.build_kv()
-        self.precentile = self.build_precentile()
+        self.percentile = self.build_percentile()
 
     def build_kv(self):
         raw_data = dict(self.data.most_common())
@@ -304,7 +371,7 @@ class Distribution:
 
         return data
 
-    def build_precentile(self):
+    def build_percentile(self):
         import numpy as np
 
         np_data = np.array(list(self.data.elements()), dtype=np.int32)
@@ -338,7 +405,13 @@ def jsonify(data):
     raise ValueError(data)
 
 
-def render_table(request_args, data, date_type=None):
+def render_table_factory(
+    request_args,
+    data,
+    date_type=None,
+    item_to_table_func=None,
+    pattern_search_func=None,
+):
     total_data_len = len(data)
     draw = request_args.get("draw")
     start = int(request_args.get("start"))
@@ -370,11 +443,29 @@ def render_table(request_args, data, date_type=None):
             "draw": draw,
             "recordsTotal": total_data_len,
             "recordsFiltered": filtered_data_len,
-            "data": [[doc.domain, doc.convert_to_md()] for doc in data_piece],
+            "data": [item_to_table_func(doc) for doc in data_piece],
         }
     )
 
 
+def doc_to_table_func(doc: Document) -> List[str]:
+    return [doc.domain, doc.convert_to_md()]
+
+
+render_table = functools.partial(
+    render_table_factory, item_to_table_func=doc_to_table_func
+)
+
+
+def span_to_table_func(span: List[Any]) -> List[str]:
+    return span
+
+
+render_table_span = functools.partial(
+    render_table_factory, item_to_table_func=span_to_table_func
+)
+
+
 def render_failure_table(request_args, data):
     data_len = len(data)
     draw = request_args.get("draw")
@@ -457,6 +548,7 @@ def upload_corpus():
         raise ValueError()
     if f:
         with temp_file_context(f) as tmp_file:
+            print(tmp_file)
             corpus_data_manager.build_from_corpus_file(tmp_file)
 
         return "OK"
@@ -467,6 +559,21 @@ def get_docs():
     return render_table(request.args, corpus_data_manager.corpus, "docs")
 
 
+@app.route("/corpus/docs/perplexity", methods=["GET"])
+def get_docs_perplexity():
+    return jsonify(corpus_data_manager.corpus_docs_text_perplexity)
+
+
+@app.route("/corpus/docs/perplexity/<string:bin>", methods=["GET"])
+def get_docs_perplexity_page(bin):
+    return render_template("docs_perplexity.html", bin=bin)
+
+
+@app.route("/corpus/docs/perplexity/<string:bin>/data", methods=["GET"])
+def get_docs_perplexity_data(bin):
+    return render_table(request.args, corpus_data_manager.corpus_docs_edge_to_docs[bin])
+
+
 @app.route("/corpus/docs/length", methods=["GET"])
 def get_docs_length():
     return jsonify(corpus_data_manager.corpus_docs_length)
@@ -641,6 +748,58 @@ def get_span_by_domain_id_and_entity_type(domain_name, span_value):
     )
 
 
+@app.route("/corpus/span/data", methods=["GET"])
+def get_span_data():
+    return render_table_span(
+        request.args,
+        [
+            (k, list(v.keys()))
+            for k, v in corpus_data_manager.span_entity_corpus.items()
+        ],
+    )
+
+
+@app.route("/corpus/span/untagged", methods=["GET"])
+def get_span_untagged_data():
+    return jsonify(corpus_data_manager.corpus_span_untagged)
+
+
+@app.route("/corpus/span/untagged/<string:text>", methods=["GET"])
+def get_span_untagged_page(text):
+    return render_template("span_untagged.html", span_untagged=text)
+
+
+@app.route("/corpus/span/untagged/<string:text>/untagged_docs", methods=["GET"])
+def get_span_untagged_page_docs(text):
+    return render_table(request.args, corpus_data_manager.corpus_span_untagged_docs[text])
+
+
+@app.route("/corpus/span/untagged/<string:text>/entity_docs", methods=["GET"])
+def get_span_untagged_page_entity_docs(text):
+    return render_table(request.args, corpus_data_manager.text_span_corpus[text])
+
+
+@app.route("/corpus/span/entity_size", methods=["GET"])
+def get_span_entity_size():
+    return jsonify(corpus_data_manager.corpus_docs_span_entity_type_size)
+
+
+@app.route("/corpus/span/entity_size/<int:size>", methods=["GET"])
+def get_span_by_entity_size(size):
+    return render_template("span_entity_size.html", entity_size=size)
+
+
+@app.route("/corpus/span/entity_size/<int:size>/list", methods=["GET"])
+def get_span_list_by_entity_size(size):
+    return render_table_span(
+        request.args,
+        [
+            (k, list(corpus_data_manager.span_entity_corpus[k].keys()))
+            for k in corpus_data_manager.corpus_docs_span_entity_type_size_to_span[size]
+        ],
+    )
+
+
 @app.route("/corpus/span/<string:span_value>", methods=["GET"])
 def get_span_by_span_value(span_value):
     return render_template("span.html", span_value=span_value)
@@ -710,6 +869,11 @@ if __name__ == "__main__":
         default=False,
         required=False,
     )
+    parser.add_argument(
+        "--use_lm",
+        help="Language model serving URL or default not use Language model",
+        default="",
+    )
 
     config = vars(parser.parse_args())
 
@@ -720,5 +884,13 @@ if __name__ == "__main__":
         es_query_func["docs"] = query_docs
         es_insert_func["docs"] = insert_corpus_into_es
 
+    use_lm_arg = config.get("use_lm", "")
+    if use_lm_arg:
+        batch_get_text_perplexity = functools.partial(
+            batch_get_text_perplexity, use_lm_arg
+        )
+    else:
+        batch_get_text_perplexity = lambda x: len(x) * [1.0]
+
     app.config["TEMPLATES_AUTO_RELOAD"] = True
-    app.run(host="0.0.0.0")
+    app.run(host="0.0.0.0", port="5000")
