@@ -1,7 +1,7 @@
 import collections
 import copy
 import itertools
-from typing import List
+from typing import List, Tuple, Optional
 
 from tokenizer_tools.tagset.offset.span import Span
 
